Package: parallelMap
Title: Unified Interface to Parallelization Back-Ends
Version: 1.5.1
Authors@R: 
    c(person(given = "Bernd",
             family = "Bischl",
             role = c("cre", "aut"),
             email = "bernd_bischl@gmx.net"),
      person(given = "Michel",
             family = "Lang",
             role = "aut",
             email = "michellang@gmail.com",
             comment = c(ORCID = "0000-0001-9754-0393")),
      person(given = "Patrick",
             family = "Schratz",
             role = "aut",
             email = "patrick.schratz@gmail.com",
             comment = c(ORCID = "0000-0003-0748-6624")))
Description: Unified parallelization framework for multiple
    back-end, designed for internal package and interactive usage.  The
    main operation is parallel mapping over lists.  Supports 'local',
    'multicore', 'mpi' and 'BatchJobs' mode.  Allows tagging of the
    parallel operation with a level name that can be later selected by the
    user to switch on parallel execution for exactly this operation.
License: BSD_2_clause + file LICENSE
URL: https://parallelmap.mlr-org.com,
        https://github.com/mlr-org/parallelMap
BugReports: https://github.com/mlr-org/parallelMap/issues
Depends: R (>= 3.0.0)
Imports: BBmisc (>= 1.8), checkmate (>= 1.8.0), parallel, stats, utils
Suggests: BatchJobs (>= 1.8), batchtools (>= 0.9.6), data.table, Rmpi,
        rpart, snow, testthat
ByteCompile: yes
Encoding: UTF-8
RoxygenNote: 7.1.0
NeedsCompilation: no
Packaged: 2021-06-27 13:21:26 UTC; pjs
Author: Bernd Bischl [cre, aut],
  Michel Lang [aut] (<https://orcid.org/0000-0001-9754-0393>),
  Patrick Schratz [aut] (<https://orcid.org/0000-0003-0748-6624>)
Maintainer: Bernd Bischl <bernd_bischl@gmx.net>
Repository: CRAN
Date/Publication: 2021-06-28 06:40:04 UTC
Built: R 4.6.0; ; 2026-04-27 06:51:22 UTC; unix
