# Generated by roxygen2: do not edit by hand

S3method(print,ParallelMapOptions)
S3method(print,RegisteredLevels)
export(parallelExport)
export(parallelGetOptions)
export(parallelGetRegisteredLevels)
export(parallelLapply)
export(parallelLibrary)
export(parallelMap)
export(parallelRegisterLevels)
export(parallelSapply)
export(parallelSource)
export(parallelStart)
export(parallelStartBatchJobs)
export(parallelStartBatchtools)
export(parallelStartLocal)
export(parallelStartMPI)
export(parallelStartMulticore)
export(parallelStartSocket)
export(parallelStop)
import(BBmisc)
import(checkmate)
import(parallel)
importFrom(stats,setNames)
importFrom(utils,getFromNamespace)
importFrom(utils,head)
importFrom(utils,tail)
